(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs/operators'), require('rxjs')) :
    typeof define === 'function' && define.amd ? define(['exports', 'rxjs/operators', 'rxjs'], factory) :
    (global = global || self, factory(global.RxjsArray = {}, global.operators, global.rxjs));
}(this, (function (exports, operators, rxjs) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    }

    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    function binarySearcher(searchValue, searchArray, property) {
      var first = 0;
      var last = searchArray.length - 1;
      var position = -1;
      var found = false;
      var middle;

      while (!found && first <= last) {
        middle = Math.round((first + last) / 2);
        var checkValue = void 0;

        if (typeof property === 'number' || property) {
          checkValue = searchArray[middle][property];
        } else {
          checkValue = searchArray[middle];
        }

        if (checkValue == searchValue) {
          found = true;
          position = middle;
        } else if (checkValue > searchValue) {
          last = middle - 1;
        } else {
          first = middle + 1;
        }
      }

      return position;
    }
    function defaultSortFn(first, second) {
      if (typeof first === 'string') {
        return first.localeCompare(second);
      }

      if (first === second) return 0;
      return first < second ? -1 : 1;
    }

    function binarySearch(searchValue, sort, property) {
      var sortFn = sort ? sort : defaultSortFn;
      return function (source) {
        return source.pipe(operators.reduce(function (acc, val) {
          if (Array.isArray(val)) {
            return __spread(acc, val);
          } else {
            return __spread(acc, [val]);
          }
        }, []), operators.map(function (accArray) {
          return __spread(accArray).sort(sortFn);
        }), operators.map(function (sortedArray) {
          return [binarySearcher(searchValue, sortedArray, property), searchValue, sortedArray];
        }));
      };
    }

    function mapDifferenceWith(input, predicate) {
      return function (value) {
        return value.filter(function (sourceValue) {
          return input.findIndex(function (checkValue) {
            if (predicate) {
              return predicate(sourceValue, checkValue);
            }

            return sourceValue === checkValue;
          }) === -1;
        });
      };
    }
    function mapDifference(checkArray, mutate) {
      if (mutate) {
        var checkSet_1 = new Set(checkArray.map(mutate));
        return function (value) {
          return __spread(new Set(value)).filter(function (x) {
            return !checkSet_1.has(mutate(x));
          });
        };
      } else {
        var checkSet_2 = new Set(checkArray);
        return function (value) {
          return __spread(new Set(value)).filter(function (x) {
            return !checkSet_2.has(x);
          });
        };
      }
    }

    function difference(input, inputModifier) {
      return function (source) {
        return rxjs.isObservable(input) ? input.pipe(operators.switchMap(function (value) {
          return source.pipe(operators.map(mapDifference(value, inputModifier)));
        })) : source.pipe(operators.map(mapDifference(input, inputModifier)));
      };
    }

    function differenceWith(input, predicate) {
      return function (source) {
        return rxjs.isObservable(input) ? input.pipe(operators.switchMap(function (inputFromSource) {
          return source.pipe(operators.map(mapDifferenceWith(inputFromSource, predicate)));
        })) : source.pipe(operators.map(mapDifferenceWith(input, predicate)));
      };
    }

    function every(predicate) {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          return value.every(function (v) {
            return predicate(v);
          });
        }));
      };
    }

    function filterEvery(predicate) {
      return function (source) {
        return source.pipe(operators.filter(function (value) {
          return value.every(function (v) {
            return predicate(v);
          });
        }));
      };
    }

    function filterSome(predicate) {
      return function (source) {
        return source.pipe(operators.filter(function (value) {
          return value.some(predicate);
        }));
      };
    }

    function find(predicate) {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          return value.find(function (v) {
            return predicate(v);
          });
        }), operators.filter(function (v) {
          return typeof v !== 'undefined';
        }));
      };
    }

    function findIndex(predicate) {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          return value.findIndex(function (v) {
            return predicate(v);
          });
        }));
      };
    }

    function flipArray() {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          return value.map(function (v) {
            return !v;
          });
        }));
      };
    }

    function indexOf(input, fromIndex, mapFn) {
      if (fromIndex === void 0) {
        fromIndex = 0;
      }

      return function (source) {
        return source.pipe(operators.map(function (value) {
          return Array.isArray(input) ? input.map(function (inputVal) {
            return mapFn ? value.map(mapFn).indexOf(inputVal, fromIndex) : value.indexOf(inputVal, fromIndex);
          }) : mapFn ? value.map(mapFn).indexOf(input, fromIndex) : value.indexOf(input, fromIndex);
        }));
      };
    }

    function mapIntersectsWith(input, predicate) {
      return function (value) {
        return value.filter(function (sourceValue) {
          return input.findIndex(function (checkValue) {
            return predicate ? predicate(sourceValue, checkValue) : sourceValue === checkValue;
          }) !== -1;
        });
      };
    }
    function mapIntersection(checkArray, mutate) {
      if (mutate) {
        var checkSet_1 = new Set(checkArray.map(mutate));
        return function (value) {
          return __spread(new Set(value)).filter(function (x) {
            return checkSet_1.has(mutate(x));
          });
        };
      } else {
        var checkSet_2 = new Set(checkArray);
        return function (value) {
          return __spread(new Set(value)).filter(function (x) {
            return checkSet_2.has(x);
          });
        };
      }
    }

    function intersects(input, inputModifier) {
      return function (source) {
        return rxjs.isObservable(input) ? input.pipe(operators.switchMap(function (value) {
          return source.pipe(operators.map(mapIntersection(value, inputModifier)));
        })) : source.pipe(operators.map(mapIntersection(input, inputModifier)));
      };
    }

    function intersectsWith(input, predicate) {
      return function (source) {
        return rxjs.isObservable(input) ? input.pipe(operators.switchMap(function (inputFromSource) {
          return source.pipe(operators.map(mapIntersectsWith(inputFromSource, predicate)));
        })) : source.pipe(operators.map(mapIntersectsWith(input, predicate)));
      };
    }

    function join(separator) {
      if (separator === void 0) {
        separator = ' ';
      }

      return function (source) {
        return source.pipe(operators.map(function (value) {
          return value.join(separator);
        }));
      };
    }

    function lastIndexOf(input, fromIndex, mapFn) {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          fromIndex = fromIndex || value.length - 1;
          return Array.isArray(input) ? input.map(function (inputVal) {
            return mapFn ? value.map(mapFn).lastIndexOf(inputVal, fromIndex) : value.lastIndexOf(inputVal, fromIndex);
          }) : mapFn ? value.map(mapFn).lastIndexOf(input, fromIndex) : value.lastIndexOf(input, fromIndex);
        }));
      };
    }

    function reverse() {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          return __spread(value).reverse();
        }));
      };
    }

    function some(predicate) {
      return function (source) {
        return source.pipe(operators.map(function (value) {
          return value.some(function (v) {
            return predicate(v);
          });
        }));
      };
    }

    function sort(fn) {
      var sortFn = fn || defaultSortFn;
      return function (source) {
        return source.pipe(operators.map(function (arr) {
          return arr.sort(function (a, b) {
            return sortFn(a, b);
          });
        }));
      };
    }

    function sortMap(mapFn, fn) {
      var sortFn = fn || defaultSortFn;
      return function (source) {
        return source.pipe(operators.map(function (arr) {
          return arr.sort(sortFn);
        }), operators.map(function (arr) {
          return arr.map(mapFn);
        }));
      };
    }

    exports.binarySearch = binarySearch;
    exports.difference = difference;
    exports.differenceWith = differenceWith;
    exports.every = every;
    exports.filterEvery = filterEvery;
    exports.filterSome = filterSome;
    exports.find = find;
    exports.findIndex = findIndex;
    exports.flipArray = flipArray;
    exports.indexOf = indexOf;
    exports.intersects = intersects;
    exports.intersectsWith = intersectsWith;
    exports.join = join;
    exports.lastIndexOf = lastIndexOf;
    exports.reverse = reverse;
    exports.some = some;
    exports.sort = sort;
    exports.sortMap = sortMap;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
